
Rem
bbdoc:
EndRem
Type VColor

	Global _current:VColor
	
	Field r:Float, g:Float, b:Float, a:Float
	
	'-----------
	'get current
	'-----------
	
	Rem
	bbdoc: Returns the current set color for drawing
	EndRem
	Function getCurrent:VColor()
		If _current
			Return _current
		Else
			Return VColor.white()
		EndIf
	EndFunction
	
	'------
	'create
	'------
	
	Rem
	bbdoc: Creates a new VColor instance
	EndRem
	Function Create:VColor( r_:Float=1.0, g_:Float=1.0, b_:Float=1.0, a_:Float=1.0 )
		Local ret_:VColor = New VColor
			If r_ > 1.0 Or g_ > 1.0 Or b_ > 1.0 Or a_ > 1.0
				ret_.initWithRGBA(r_, g_, b_, a_)
			Else
				ret_.initWithRGBAf(r_, g_, b_, a_)
			EndIf
		Return ret_
	EndFunction
	
	Method initWithRGBA( r_:Int, g_:Int, b_:Int, a_:Int)
		r = r_/255.0
		g = g_/255.0
		b = b_/255.0
		a = a_/255.0
	EndMethod
	
	Method initWithRGBAf( r_:Float, g_:Float, b_:Float, a_:Float )
		r = r_
		g = g_
		b = b_
		a = a_
	EndMethod
	
	'----------
	'accsessors
	'----------
	
	Rem
	bbdoc: Set the RGBA values in int 0.0-255.0
	EndRem
	Method setRGBA( r_:Int, g_:Int, b_:Int, a_:Int )
		r = r_/255.0
		g = g_/255.0
		b = b_/255.0
		a = a_/255.0
	EndMethod
	
	Rem
	bbdoc: Get the RGBA values in int 0.0-255.0
	EndRem
	Method getRGBA( r_:Int Var, g_:Int Var, b_:Int Var, a_:Int Var )
		r_ = r*255.0
		g_ = g*255.0
		b_ = b*255.0
		a_ = a*255.0
	EndMethod
	
	Rem
	bbdoc: Set the RGBA values in float 0.0-1.0
	EndRem
	Method setRGBAf( r_:Float, g_:Float, b_:Float, a_:Float )
		r = r_
		g = g_
		b = b_
		a = a_
	EndMethod
	
	Rem
	bbdoc: Get the RGBA values in float 0.0-1.0
	EndRem
	Method getRGBAf( r_:Float Var, g_:Float Var, b_:Float Var, a_:Float Var )
		r_ = r
		g_ = g
		b_ = b
		a_ = a
	EndMethod
	
	'---
	'set
	'---
	
	Rem
	bbdoc: Set the color for drawing
	EndRem
	Method set()
		VColor._current = Self
		glColor4f(r, g, b, a)
	EndMethod
	
	'----
	'copy
	'----
	
	Rem
	bbdoc: Returns a deep copy of of the object
	EndRem
	Method copy:VColor()
		Return VColor.Create(r, g, b, a)
	EndMethod
	
	'------
	'string
	'------
	
	Method toString:String()
		Return "{"+r+","+g+","+b+","+a+"}"
	EndMethod
	
	'------
	'colors
	'------
	
	Function white:VColor()
		Return VColor.Create(1.0, 1.0, 1.0, 1.0)
	EndFunction
	
	Function black:VColor()
		Return VColor.Create(0.0, 0.0, 0.0, 1.0)
	EndFunction
	
	Function grey:VColor()
		Return VColor.Create(0.5, 0.5, 0.5, 1.0)
	EndFunction
	
	Function red:VColor()
		Return VColor.Create(1.0, 0.0, 0.0, 1.0)
	EndFunction
	
	Function green:VColor()
		Return VColor.Create(0.0, 1.0, 0.0, 1.0)
	EndFunction
	
	Function blue:VColor()
		Return VColor.Create(0.0, 0.0, 1.0, 1.0)
	EndFunction
	
	Function clear:VColor()
		Return VColor.Create(0.0, 0.0, 0.0, 0.0)
	EndFunction

EndType