'
'
'

Rem
bbdoc: OpenGL FBO-ext
EndRem
Type VFBO

	Field _frame_buffer :Int
	Field _render_buffer:Int

	Field _texture      :Int	
	Field _width        :Int
	Field _height       :Int
	
	Field _bound        :Int
	
	Field _old_matrix_pj:Float[16] 'projection
	Field _old_matrix_mv:Float[16] 'modelview
	Field _old_vp       :Float[4 ] 'viewport
	Field _matrix_pj    :Float[16] 'projection
	Field _matrix_mv    :Float[16] 'modelview
	Field _vp           :Float[4 ] 'viewport
	
	'------
	'create
	'------
	
	Function Create:VFBO( texture_:Int, width_:Int, height_:Int )
		Local ret_:VFBO = New VFBO
			ret_.initWithTexture(texture_, width_, height_)
		Return ret_
	EndFunction
	
	Method initWithTexture( texture_:Int, width_:Int, height_:Int )
		Self.setTexture(texture_, width_, height_)
		
		glGenFramebuffersEXT (1, Varptr(_frame_buffer ))
		glGenRenderbuffersEXT(1, Varptr(_render_buffer))
	
		If (_frame_buffer=-1 And _render_buffer=-1 And _texture) FBOError("Error #01") ; Return
		
		glBindFramebufferEXT          (GL_FRAMEBUFFER_EXT, _frame_buffer )
		glFramebufferTexture2DEXT     (GL_FRAMEBUFFER_EXT, GL_COLOR_ATTACHMENT0_EXT, GL_TEXTURE_2D, _texture, 0)
		glBindRenderbufferEXT         (GL_RENDERBUFFER_EXT, _render_buffer )
		glRenderbufferStorageEXT      (GL_RENDERBUFFER_EXT, GL_DEPTH_COMPONENT24, width_, height_)
		glFramebufferRenderbufferEXT  (GL_FRAMEBUFFER_EXT, GL_DEPTH_ATTACHMENT_EXT, GL_RENDERBUFFER_EXT, _render_buffer)
		
		FBOError(checkFBOError())
	EndMethod
	
	'---------------------
	'TImage implimentation
	'---------------------
	
	Function withTImage:VFBO( image_:TImage, frame_:Int=0 )
		If Not (image_) FBOError("No image") ; Return Null
		Return VFBO.Create(TGLImageFrame(image_.Frame(frame_)).name, image_.width, image_.height)
	EndFunction
	
	'-----
	'ortho
	'-----
	
	Rem
	bbdoc: The default matrix set for the FBO
	EndRem
	Method setUpDefaultMatrix()
		'save old matrix
		Self.storeMatrix()
	
		'create new matrix
		glMatrixMode(GL_PROJECTION)
		glLoadIdentity()
		glOrtho(0,_width, 0, _height,-1,1)
		glMatrixMode(GL_MODELVIEW)
		glLoadIdentity()
		glViewport(0,0,_width,_height)
		
		'save new and reload old matrix
		glGetFloatv(GL_VIEWPORT, _vp)
		glGetFloatv(GL_PROJECTION_MATRIX, _matrix_pj);
		glGetFloatv(GL_MODELVIEW_MATRIX,  _matrix_mv);
		
		Self.resetMatrix()
	EndMethod
	
	'------
	'matrix
	'------
	
	Rem
	bbdoc: Set a custom matrix for FBO rendering
	EndRem
	Method setMatrix( matrix_:Float[], mode_:Int )
		If matrix_.length <> 16 Return
		Select(mode_)
			Case GL_PROJECTION
				_matrix_pj = matrix_
			Case GL_MODELVIEW
				_matrix_mv = matrix_
			Default
				WriteStdout("[VFBO] Error setting unknwon matrix mode!~n")
		EndSelect
	EndMethod
	
	Rem
	bbdoc: Automaticaly called in "bind"
	EndRem
	Method storeMatrix()
		'save old matrix
		glGetFloatv(GL_VIEWPORT, _old_vp)
		glGetFloatv(GL_PROJECTION_MATRIX, _old_matrix_pj)
		glGetFloatv(GL_MODELVIEW_MATRIX,  _old_matrix_mv)
		
		'push new loaded matrix
		glPushMatrix()
		
		glMatrixMode(GL_PROJECTION)
		glLoadMatrixf(_matrix_pj)
		glMatrixMode(GL_MODELVIEW)
		glLoadMatrixf(_matrix_mv)
		glViewport(_vp[0], _vp[1], _vp[2], _vp[3])
	EndMethod
	
	Rem
	bbdoc: Automaticaly called in "unbind"
	EndRem
	Method resetMatrix()
		'pop new matrix
		glPopMatrix()
	
		'reload old matrix
		glMatrixMode(GL_PROJECTION)
		glLoadMatrixf(_old_matrix_pj)
		glMatrixMode(GL_MODELVIEW)
		glLoadMatrixf(_old_matrix_mv)
		glViewport(_old_vp[0], _old_vp[1], _old_vp[2], _old_vp[3])
	EndMethod
	
	'----------
	'accsessors
	'----------
	
	Rem
	bbdoc: Attach a texture to the FBO
	EndRem
	Method setTexture( texture_:Int, width_:Int, height_:Int )
		_texture = texture_
		
		_width = width_
		_height = height_
		
		Self.detachTexture()
		Self.attachTexture()
		
		Self.setUpDefaultMatrix()
	EndMethod
	
	Rem
	bbdoc: Get the glTexture the FBO uses to render in
	EndRem
	Method getTexture:Int()
		Return _texture
	EndMethod
	
	'-----------
	'bind/unbind
	'-----------
	
	Rem
	bbdoc: Bind the FBO for rendering
	about: if ortho_ = 1 setOrtho() will be called to set up new glOrtho
	EndRem
	Method bind()
		If _bound = 1 Return
		Self.storeMatrix()
		
		glBindFramebufferEXT(GL_FRAMEBUFFER_EXT, _frame_buffer)
				
		_bound = 1
	EndMethod
	
	Rem
	bbdoc: Unbind the FBO
	about: if ortho_ = 1 resetOrtho() will be called to reset the glOrtho to the graphicsWidth/Height
	EndRem
	Method unbind( ortho_:Int=1 )
		If _bound = 0 Return
		
		glBindFramebufferEXT(GL_FRAMEBUFFER_EXT, 0)
		Self.resetMatrix()
		
		_bound = 0
	EndMethod
	
	'-------
	'texture
	'-------
	
	Method attachTexture()
		Self.bind()
		glFramebufferTexture2DEXT(GL_FRAMEBUFFER_EXT, GL_COLOR_ATTACHMENT0_EXT, GL_TEXTURE_2D, _texture, 0)	
		FBOError(checkFBOError())
		Self.unbind()
	EndMethod
	
	Method detachTexture()
		Self.bind()
		glFramebufferTexture2DEXT(GL_FRAMEBUFFER_EXT, GL_COLOR_ATTACHMENT0_EXT, GL_TEXTURE_2D, 0, 0)	
		FBOError(checkFBOError())
		Self.unbind()
	EndMethod
	
	'delete
	Method deinit()
		glDeleteFramebuffersEXT (1, Varptr(_frame_buffer))
		glDeleteRenderbuffersEXT(1, Varptr(_frame_buffer))
		
		_bound = 0
	EndMethod
	
	Method Delete()
		unbind(1)
		deinit()
	EndMethod

EndType

Private
Function FBOError( error_:String )
	If Not error_ Return
	WriteStdout("[FBO EXT] " + error_+"~n")
	?debug
		Throw error_
	?
EndFunction
Function checkFBOError:String()
	Local error_string_:String
	Select glCheckFramebufferStatusEXT ( GL_FRAMEBUFFER_EXT )	
		Case	GL_FRAMEBUFFER_INCOMPLETE_ATTACHMENT_EXT
			error_string_	= "GL_FRAMEBUFFER_INCOMPLETE_ATTACHMENT_EXT"
		Case	GL_FRAMEBUFFER_INCOMPLETE_MISSING_ATTACHMENT_EXT
			error_string_	= "GL_FRAMEBUFFER_INCOMPLETE_MISSING_ATTACHMENT_EXT"
		Case	GL_FRAMEBUFFER_INCOMPLETE_DIMENSIONS_EXT
			error_string_	= "GL_FRAMEBUFFER_INCOMPLETE_DIMENSIONS_EXT"	
		Case	GL_FRAMEBUFFER_INCOMPLETE_FORMATS_EXT
			error_string_	= "GL_FRAMEBUFFER_INCOMPLETE_FORMATS_EXT"	
		Case	GL_FRAMEBUFFER_INCOMPLETE_DRAW_BUFFER_EXT
			error_string_	= "GL_FRAMEBUFFER_INCOMPLETE_DRAW_BUFFER_EXT"	
		Case	GL_FRAMEBUFFER_INCOMPLETE_READ_BUFFER_EXT
			error_string_	= "FGL_RAMEBUFFER_INCOMPLETE_READ_BUFFER_EXT"
		Case	GL_FRAMEBUFFER_UNSUPPORTED_EXT
			error_string_	= "GL_FRAMEBUFFER_UNSUPPORTED_EXT"
	EndSelect
	Return error_string_
EndFunction
Public