'
'
'

Rem
bbdoc: VFont default glyph array size (ASCI)
EndRem
Const VFontDefaultGlyphCount:Int = 256

Rem
bbdoc: Visuals font class for loading and drawing fonts
EndRem
Type VFont Abstract

	Field _size:Float
	Field _attr:Int
	
	Field _glyphes:VGlyph[]
	
	'------
	'create
	'------
	
	Method init()
		_glyphes = New VGlyph[256]
		Self.initWithSize(0.0)
	EndMethod
	
	Rem
	bbdoc: Overwrite
	EndRem
	Method initWithSize( size_:Float ) Abstract
	
	'----------
	'accsessors
	'----------
	
	Rem
	bbdoc: Set the fonts size
	about: Calls 'loadWithSize()'
	EndRem
	Method setSize( size_:Float )
		_size = Max(1.0, size_)
		loadWithSize(_size, _attr)
	EndMethod
	
	Rem
	bbdoc: Returns the fonts size
	EndRem
	Method getSize:Float()
		Return _size
	EndMethod
	
	Rem
	bbdoc: Set font attributes
	about: Calls 'LoadWithSize()'
	EndRem
	Method setAttributes(  attr_:Int )
		_attr = attr_
		loadWithSize(_size, _attr)
	EndMethod
	
	Rem
	bbdoc: Returns the fonts attributes
	EndRem
	Method getAttributes:Int()
		Return _attr
	EndMethod
	
	'----
	'load
	'----
	
	Rem
	bbdoc: Load the font with given size and sttributes
	about: Returns the loaded size
	EndRem
	Method loadWithSize:Float( size_:Float, attr_:Int ) Abstract
	
	'-----
	'glyph
	'-----
	
	Rem
	bbdoc: Returns the glyph with given ASCI code
	EndRem
	Method getGlyph:VGlyph( asci_:Int ) Abstract
	
	Rem
	bbdoc: Returns a array with all available glyphes of the font
	EndRem
	Method getGlyphes:VGlyph[]()
		Return _glyphes
	EndMethod

EndType

Rem
bbdoc: VGlyph
EndRem
Type VGlyph Abstract

	Field _width :Float
	Field _height:Float
	
	'------
	'create
	'------
	
	Method init()
		setSize(0.0, 0.0)
	EndMethod
	
	'----------
	'accsessors
	'----------
	
	Rem
	bbdoc: Set the width and height of the glyph
	EndRem
	Method setSize( width_:Float, height_:Float )
		_width = Max(0.0, width_  )
		_height = Max(0.0, height_)
	EndMethod
	
	Rem
	bbdoc: Get the width and height of the glyph
	EndRem
	Method getSize( width_:Float Var, height_:Float Var )
		width_ = _width
		height_= _height
	EndMethod
	
	'------
	'render
	'------
	
	Rem
	bbdoc: Draw the glyph at the given point with specified scale factor
	about: Calls 'drawInRect(x, y, widht*scale, height*scale )'
	EndRem
	Method drawAtPoint( x_:Float, y_:Float, scale_:Float=1.0 )
		drawInRect(x_, y_, _width*scale_, _height*scale_)
	EndMethod
	
	Rem
	bbdoc: Draws the glyph scaled in rect (fit the rect)
	EndRem
	Method drawInRect ( x_:Float, y_:Float, width_:Float, height_:Float ) Abstract

EndType
