
Rem
bbdoc: Horizontal
EndRem
Const ORIENTATION_HORIZONTAL:Int = 0

Rem
bbdoc: Vertical
EndRem
Const ORIENTATION_VERTICAL  :Int = 1

Rem
bbdoc: VGradient
EndRem
Type VGradient Abstract

	Field _markers:TList
	
	Rem
	bbdoc: Draw the gradient
	EndRem
	Method drawInRect( x_:Float, y_:Float, width_:Float, height_:Float ) Abstract
	
	Rem
	bbdoc: Draw the gradient into a StdRect
	EndRem
	Method drawInStdRect( rect_:StdRect )
		Self.drawInRect(rect_.origin.x, rect_.origin.y, rect_.size.width, rect_.size.height)
	EndMethod
	
	Rem
	bbdoc: Draws the Gradient at 0.0, 0.0 with a specific size
	EndRem
	Method drawWithStdSize( size_:StdSize )
		Self.drawInRect(0.0, 0.0, size_.width, size_.height)
	EndMethod

EndType

Rem
bbdoc: Linear gradient class
about: 
EndRem
Type VLinearGradient Extends VGradient

	Field _vector:Float[5]
	Field _count:Int
	Field _orientation:Byte

	'------
	'create
	'------

	Rem
	bbdoc: Creates a new empty Linear gradient
	about: 
	EndRem
	Function Create:VLinearGradient()
		Local ret_:VLinearGradient = New VLinearGradient
			ret_.init()
		Return ret_
	EndFunction
	
	Rem
	bbdoc: 
	about: 
	EndRem
	Method init()
		_markers = CreateList()
		_orientation = 0 'horizontal
		_count = 0
	EndMethod
	
	'----------
	'accsessors
	'----------
	
	Rem
	bbdoc: Set the gradients orientation
	about: Get using #getOrientation
	EndRem
	Method setOrientation( orientation_:Int=0 )
		_orientation = orientation_
	EndMethod
	
	Rem
	bbdoc: Returns the gradients orientation
	about: Set using #setOrientation()
	EndRem
	Method getOrientation:Int()
		Return _orientation
	EndMethod

	'---
	'add
	'---

	Rem
	bbdoc: Adds a colorstop to the gradient
	about: The position is relative in % (0.0 - 1.0)<br>All colors are float values
	EndRem
	Method addColorStop:VColorStop( pos_:Float, r_:Float, g_:Float, b_:Float, a_:Float=1.0 )
		If pos_ < 0.0 pos_ = 0.0
		If pos_ > 1.0 pos_ = 1.0
		Local marker_:VColorStop = VColorStop.Create(pos_, r_, g_, b_, a_)
		_markers.addLast(marker_)
		marker_._link = _markers.lastLink()
		_markers.sort(True, VColorStop.msortFunc)
		_recalcVector()
		Return marker_
	EndMethod
	
	Rem
	bbdoc: Returns the color stop on the specified position (0-1) or NULL if it does not exist
	EndRem
	Method getColorStop:VColorStop( pos_:Float )
		If pos_ < 0.0 Or pos_ > 1.0 Return Null
		For Local stop_:VColorStop = EachIn(_markers)
			If stop_._pos >= pos_-0.0001 And stop_._pos <= pos_+0.0001 Return stop_
		Next
		Return Null
	EndMethod
	
	'------
	'method
	'------
	
	Method recalc()
		_recalcVector()
	EndMethod	
	
	'----
	'draw
	'----
	
	Rem
	bbdoc: Draw the gradient with given width and height
	about: Drawing gradients is very fast
	EndRem
	Method drawInRect( x_:Float, y_:Float, width_:Float, height_:Float )
		VStartDraw(GL_QUAD_STRIP, x_, y_)
			Select(_orientation)
				Case ORIENTATION_HORIZONTAL
					For Local i:Int = 0 Until _count Step 5
						glColor4f(_vector[i+1], _vector[i+2], _vector[i+3], _vector[i+4])
						glVertex3f(_vector[i]*width_, 0.0, 1.0 )
						glVertex3f(_vector[i]*width_, height_, 1.0 )
					Next
				Case ORIENTATION_VERTICAL
					For Local i:Int = 0 Until _count Step 5
						glColor4f(_vector[i+1], _vector[i+2], _vector[i+3], _vector[i+4])
						glVertex3f(0.0, _vector[i]*height_, 1.0 )
						glVertex3f(width_, _vector[i]*height_, 1.0 )
					Next
			EndSelect
		VEndDraw()
	EndMethod
	
	'-------
	'private
	'-------
	
	Method _recalcVector()
		_vector = New Float[_markers.count()*5]
		Local pos_:Int = 0
		
		For Local marker_:VColorStop = EachIn(_markers)
			_vector[pos_  ] = marker_._pos
			_vector[pos_+1] = marker_._r
			_vector[pos_+2] = marker_._g
			_vector[pos_+3] = marker_._b
			_vector[pos_+4] = marker_._a
			pos_:+ 5
		Next
		_count = pos_
	EndMethod	

EndType

Rem
bbdoc: 
EndRem
Type VColorStop
	Field _pos:Float 'pos in % (0-1)
	Field _r:Float
	Field _g:Float
	Field _b:Float
	Field _a:Float
	Field _link:TLink
	
	Function Create:VColorStop( pos_:Float, r_:Float, g_:Float, b_:Float, a_:Float )
		Local ret_:VColorStop = New VColorStop
			ret_.initWithPos(pos_, r_, g_, b_, a_)
		Return ret_
	EndFunction
	
	Method initWithPos( pos_:Float, r_:Float, g_:Float, b_:Float, a_:Float )
		_pos = pos_
		_r = r_
		_g = g_
		_b = b_
		_a = a_
	EndMethod
	
	'----------
	'accsessors
	'----------
	
	Method getPosition:Float()
		Return _pos
	EndMethod
	
	Method setPosition( pos_:Float )
		_pos = pos_
	EndMethod
	
	'------
	'remove
	'------
	
	Method remove()
		If _link _link.remove()
	EndMethod
	
	'----
	'copy
	'----
	
	Method copy:VColorStop()
		Return VColorStop.Create(_pos, _r, _g, _b, _a)
	EndMethod
	
	'-------
	'private
	'-------
	
	Function msortFunc:Int( o1:Object, o2:Object )
		Local m1:VColorStop = VColorStop(o1)
		Local m2:VColorStop = VColorStop(o2)
		If m1._pos >= m2._pos
			Return 1
		Else
			Return 0
		EndIf
	EndFunction
	
EndType


'--

Function VStartDraw( mode_:Int, x_:Float, y_:Float, z_:Float=0.0 )
	Local scaleX:Float, scaleY:Float
	Local originX:Float, originY:Float
	Local handleX:Float, handleY:Float
	GetHandle(handleX, handleY)
	GetScale(scaleX, scaleY)
	GetOrigin(originX, originY)
	Visuals.disableTexture()
	glPushMatrix()
	glTranslatef(x_+originX-(x_*(scaleX-1.0)), y_+originY-(y_*(scaleY-1.0)), 0.0)
	glScalef(scaleX, scaleY, 1.0)
	glRotatef(GetRotation(), 0.0, 0.0, 1.0)
	glTranslatef(-handleX, -handleY, 0.0)
		glBegin( mode_ )
		VColor.getCurrent().set()
EndFunction

Function VEndDraw()
		glEnd()
	glPopMatrix()
	Visuals.enableTexture()
EndFunction
Public